from enum import Enum


class KindSel(str, Enum):
    ALL = 'all'
    MODE = 'mode'


def all_knn_undersampling(
        df, target_col: str, result: list[str] | None = None, n_neighbors: int = 4,
        kind_sel: KindSel | str = KindSel.ALL):
    """
    Applies All KNN (AKNN) under-sampling to balance the dataset by removing samples that are misclassified by all of
    their k nearest neighbors. Allows setting the selection type using Enum.

    :param target_col: Name of the target column to balance.
    :param result: List to store the resulting column names after under-sampling.
    :param n_neighbors: Number of nearest neighbors to consider. Default is 4.
    :param kind_sel: Selection method to use, specified as an Enum (KindSel.ALL or KindSel.MODE) or as a string.
    """
    import pandas as pd
    from imblearn.under_sampling import AllKNN

    # Convert kind_sel to KindSel Enum if it's passed as string
    if isinstance(kind_sel, str):
        try:
            kind_sel = KindSel[kind_sel.upper()]
        except KeyError:
            raise ValueError(f"'{kind_sel}' is not a valid KindSel value. Use 'ALL' or 'MODE'.")

    # Instantiate AllKNN with additional parameters
    all_knn = AllKNN(n_neighbors=n_neighbors, kind_sel=kind_sel.value)

    # Extract features and target
    X = df.drop(columns=[target_col])
    y = df[target_col]

    # Apply All KNN (AKNN) under-sampling
    x_resampled, y_resampled = all_knn.fit_resample(X, y)

    # Create a new DataFrame with resampled data
    df_resampled = pd.concat(
        [pd.DataFrame(x_resampled, columns=X.columns), pd.DataFrame(y_resampled, columns=[target_col])], axis=1)

    # Add resampled columns to the result list
    if result is not None:
        result.extend(df_resampled.columns.tolist())

    return df_resampled


all_knn_undersampling.worker_required = True
