from enum import Enum


class Algorithms(str, Enum):
    XGBOOST_CLASSIFICATION = "XGBoost Classification"
    LIGHTGBM_CLASSIFICATION = "LightGBM Classification"
    XGBOOST_REGRESSION = "XGBoost Regression"
    LIGHTGBM_REGRESSION = "LightGBM Regression"


def feature_importance(df, target_column: str, algorithm: Algorithms, random_state: int, test_size: float = 0.2):
    """
    Selects important features based on the specified machine learning algorithm (XGBoost or LightGBM) for either classification or regression.
    Allows the user to specify the train-test split ratio.

    :param df: DataFrame containing the dataset.
    :param target_column: The name of the target column.
    :param algorithm: The machine learning algorithm to use for feature selection.
    :param random_state: Integer seed for random number generator.
    :param test_size: The proportion of the dataset to include in the test split.
    """
    import pandas as pd
    from sklearn.model_selection import train_test_split
    import xgboost as xgb
    import lightgbm as lgb

    X = df.drop(columns=[target_column])
    y = df[target_column]

    x_train, x_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=random_state)

    if algorithm in [Algorithms.XGBOOST_CLASSIFICATION, Algorithms.XGBOOST_REGRESSION]:
        model = xgb.XGBRegressor(random_state=random_state) if algorithm == Algorithms.XGBOOST_REGRESSION else xgb.XGBClassifier(random_state=random_state)
    elif algorithm in [Algorithms.LIGHTGBM_CLASSIFICATION, Algorithms.LIGHTGBM_REGRESSION]:
        model = lgb.LGBMRegressor(random_state=random_state) if algorithm == Algorithms.LIGHTGBM_REGRESSION else lgb.LGBMClassifier(random_state=random_state)
    else:
        raise ValueError("Unsupported algorithm selected.")

    model.fit(x_train, y_train)
    feature_importances = model.feature_importances_

    # Append importance score to feature names and create a DataFrame
    feature_names_with_importance = {f"{feature}_{importance:.4f}": importance for feature, importance in zip(X.columns, feature_importances)}
    sorted_feature_names_with_importance = dict(sorted(feature_names_with_importance.items(), key=lambda item: item[1], reverse=True))

    selected_features_df = pd.DataFrame.from_dict(sorted_feature_names_with_importance, orient='index', columns=['Importance'])

    return selected_features_df


feature_importance.worker_required = True
