from enum import Enum


class AlgorithmType(str, Enum):
    REGRESSION = "regression"
    CLASSIFICATION = "classification"


def impute_missing_lgbm(df, target_col: str, alg_type: AlgorithmType, random_state: int = 42):
    """
    Imputes missing values in a specified column using LightGBM, based on the specified algorithm type (regression or classification).

    Parameters:
    :param target_col: The name of the column with missing values to impute.
    :param alg_type: The type of algorithm to use ('regression' or 'classification').
    :param random_state: Integer seed for random number generator to ensure reproducibility.
    """
    from lightgbm import LGBMRegressor, LGBMClassifier

    # Separate the data into those with and without missing values for the target column
    df_missing = df[df[target_col].isna()]
    df_not_missing = df.dropna(subset=[target_col])

    # Prepare the training data and the data where the target is missing
    x_train = df_not_missing.drop(columns=[target_col])
    y_train = df_not_missing[target_col]
    x_missing = df_missing.drop(columns=[target_col])

    # Initialize and train the LightGBM model based on the algorithm type
    if alg_type == AlgorithmType.REGRESSION:
        model = LGBMRegressor(random_state=random_state)
    elif alg_type == AlgorithmType.CLASSIFICATION:
        model = LGBMClassifier(random_state=random_state)
    else:
        raise ValueError("Unsupported algorithm type. Choose 'regression' or 'classification'.")

    model.fit(x_train, y_train)

    # Predict the missing values
    predictions = model.predict(x_missing)

    # Update the DataFrame with the imputed values
    df.loc[df[target_col].isna(), target_col] = predictions

    return df


impute_missing_lgbm.worker_required = True
