from enum import Enum


class EstimatorType(str, Enum):
    K_NEAREST_NEIGHBORS = "k_nearest_neighbors"
    DECISION_TREE = "decision_tree"
    RANDOM_FOREST = "random_forest"


def instance_hardness_threshold_undersampling(
        df, target_col: str, result: list[str] | None,
        estimator: EstimatorType | None = EstimatorType.K_NEAREST_NEIGHBORS, random_state: int | None = None):
    """
    Applies Instance Hardness Threshold (IHT) under-sampling to balance the dataset by removing samples with hardness
    scores below a certain threshold.

    :param target_col: Name of the target column to balance.
    :param result: List to store the resulting column names after under-sampling.
    :param estimator: The estimator used to compute the hardness scores. If None, it uses a 3-nearest neighbors.
                      Default is EstimatorType.K_NEAREST_NEIGHBORS.
    :param random_state: Controls the randomness of the estimator. Passed directly to the InstanceHardnessThreshold constructor.
    """

    import pandas as pd
    from sklearn.neighbors import KNeighborsClassifier
    from sklearn.tree import DecisionTreeClassifier
    from sklearn.ensemble import RandomForestClassifier
    from imblearn.under_sampling import InstanceHardnessThreshold

    estimator_dict = {
        EstimatorType.K_NEAREST_NEIGHBORS: KNeighborsClassifier(n_neighbors=3),
        EstimatorType.DECISION_TREE: DecisionTreeClassifier(),
        EstimatorType.RANDOM_FOREST: RandomForestClassifier()
    }

    selected_estimator = estimator_dict.get(estimator, KNeighborsClassifier(n_neighbors=3))

    # Instantiate InstanceHardnessThreshold with the selected estimator and random_state
    iht = InstanceHardnessThreshold(estimator=selected_estimator, random_state=random_state)

    # Extract features and target
    X = df.drop(columns=[target_col])
    y = df[target_col]

    # Apply Instance Hardness Threshold (IHT) under-sampling
    x_resampled, y_resampled = iht.fit_resample(X, y)

    # Create a new DataFrame with resampled data
    df_resampled = pd.concat(
        [pd.DataFrame(x_resampled, columns=X.columns), pd.DataFrame(y_resampled, columns=[target_col])], axis=1)

    # Add resampled columns to the result list
    if result is not None:
        result.extend(df_resampled.columns.tolist())

    return df_resampled


instance_hardness_threshold_undersampling.worker_required = True
