from enum import Enum


class ModelType(str, Enum):
    REGRESSION = "RandomForestRegressor"
    CLASSIFICATION = "RandomForestClassifier"


def rfe(df, target_col: str, n_features_to_select: int, model_type: ModelType, prefix: str):
    """
    Applies Recursive Feature Elimination (RFE) to select top N important features.
    :param target_col: Name of the target column.
    :param n_features_to_select: Number of features to select.
    :param model_type: Type of model to use for feature importance, e.g., REGRESSION or CLASSIFICATION.
    :param prefix: Prefix for the new columns indicating if a feature is selected (1) or not (0).
    """
    import pandas as pd
    from sklearn.feature_selection import RFE
    from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier

    x = df.drop(columns=[target_col])
    y = df[target_col]

    # Choose the model based on the input
    if model_type == ModelType.REGRESSION:
        estimator = RandomForestRegressor()
    elif model_type == ModelType.CLASSIFICATION:
        estimator = RandomForestClassifier()
    else:
        raise ValueError(f"Unknown model type: {model_type}")

    selector = RFE(estimator, n_features_to_select=n_features_to_select)
    selector = selector.fit(x, y)

    # Create a DataFrame to indicate selected features
    selected_features = pd.Series(selector.support_, index=x.columns, name='selected')
    df[prefix + '_selected'] = selected_features.apply(lambda f: 1 if f else 0)

    return df


rfe.worker_required = True
rfe.supported_engines = ['pandas']
