def rfm(df, id_col: str, recency_col: str, frequency_col: str, monetary_col: str, unique: bool = False):

    """
    Calculates Recency, Frequency, and Monetary (RFM) metrics for customer segmentation.
    
    :param id_col: Name of the column containing unique customer identifiers.
    :param recency_col: Name of the column containing the date of the transaction.
    :param frequency_col: Name of the column containing the frequency of transactions.
    :param monetary_col: Name of the column containing the monetary value of transactions.
    :param unique: Options which are indicating whether to count unique transactions or all transactions.
    """
    
    import pandas as pd
    import datetime as dt

    # Date of most recent order:
    df[recency_col] = pd.to_datetime(df[recency_col])
    today_date = df[recency_col].max() + dt.timedelta(days=2)

    # If frequncy is calculated on customer ID:
    if id_col == frequency_col:
        df['frequency'] = df[id_col]
        frequency_col = 'frequency'

    # Definin RFM metrics:
    if unique == True:
        rfm = df.groupby(id_col).agg({recency_col: lambda recency: (today_date - recency.max()).days,
                                      frequency_col: lambda frequency: frequency.nunique(),
                                      monetary_col: 'sum'}).reset_index()
    else:
        rfm = df.groupby(id_col).agg({recency_col: lambda recency: (today_date - recency.max()).days,
                                      frequency_col: 'count',
                                      monetary_col: 'sum'}).reset_index()
        
    rfm.columns = [id_col, 'recency', 'frequency', 'monetary']

    # Making rfm scores on 5-point scale by using "qcut" method:
    rfm['recency_score'] = pd.qcut(rfm['recency'].rank(method='first'), 5, labels=[5, 4, 3, 2, 1])
    rfm['frequency_score'] = pd.qcut(rfm['frequency'].rank(method='first'), 5, labels=[1, 2, 3, 4, 5])
    rfm['monetary_score'] = pd.qcut(rfm['monetary'].rank(method='first'), 5, labels=[1, 2, 3, 4, 5])

    # Defining RF Score:
    rfm['RF_SCORE'] = rfm['recency_score'].astype(str) + rfm['frequency_score'].astype(str)
    rfm['RF_SCORE'].head(10)

    # Making the segmentation by mapping:
    seg_map = {
        r'[1-2][1-2]': 'hibernating',
        r'[1-2][3-4]': 'at_Risk',
        r'[1-2]5': 'cant_loose',
        r'3[1-2]': 'about_to_sleep',
        r'33': 'need_attention',
        r'[3-4][4-5]': 'loyal_customers',
        r'41': 'promising',
        r'51': 'new_customers',
        r'[4-5][2-3]': 'potential_loyalists',
        r'5[4-5]': 'champions'
    }
    rfm['segment'] = rfm['RF_SCORE'].replace(seg_map, regex=True)

    # Identify common columns
    common_columns = df.columns.intersection(rfm.columns).tolist()
    common_columns.remove(id_col)  # Keep the merge key

    # Drop common columns from df
    df = df.drop(columns=common_columns)

    # Merge rfm with main DataFrame
    df = pd.merge(df, rfm, on = id_col)

    return df

rfm.worker_required = True
rfm.supported_engines = ['pandas']