from enum import Enum


class TransformationOptions(str, Enum):
    QUANTILE = "Quantile Transformation"
    YEO_JOHNSON = "Yeo-Johnson Transformation"


def transformation(df, numeric_col_list: list[str] | None = None,
                   transformation_option: TransformationOptions = TransformationOptions.QUANTILE,
                   result_col_names: list[str] | None = None):
    """
    Applies specified transformation (Quantile or Yeo-Johnson) to numeric columns of a DataFrame.

    Parameters:
    :param numeric_col_list: List of numeric column names to transform. If None, all numeric columns will be considered.
    :param transformation_option: Type of transformation to apply, specified as a member of the TransformationOptions enum.
    :param result_col_names: Optional list of new column names for the transformed data.
                        If None, original column names will be appended with '_transformed'.

    Returns:
    - The DataFrame with specified columns transformed.
    """
    from sklearn.preprocessing import QuantileTransformer, PowerTransformer
    import numpy as np

    # If no specific columns provided, use all numeric columns
    if numeric_col_list is None:
        numeric_col_list = df.select_dtypes(include=np.number).columns.tolist()

    # If no result column names are provided, generate them
    if result_col_names is None or len(result_col_names) != len(numeric_col_list):
        result_col_names = [f"{col}_transformed" for col in numeric_col_list]

    # Select the transformation method
    if transformation_option == TransformationOptions.QUANTILE:
        transformer = QuantileTransformer()
    elif transformation_option == TransformationOptions.YEO_JOHNSON:
        transformer = PowerTransformer(method='yeo-johnson')
    else:
        raise ValueError(f"Unknown transformer option: {transformation_option}")

    # Apply the transformation to each specified column and store in the specified result column name
    for col, new_col in zip(numeric_col_list, result_col_names):
        # Ensure the column exists in DataFrame
        if col in df.columns:
            df[new_col] = transformer.fit_transform(df[[col]])
        else:
            print(f"Warning: Column '{col}' not found in DataFrame. Transformation skipped.")

    return df


transformation.worker_required = True
