from enum import Enum


class FeatureSelectionMethod(str, Enum):
    CHI2 = "Chi-square test for categorical target"
    F_REGRESSION = "F-test for continuous target"


def feature_selection_helper(x, y, k, method):
    from sklearn.feature_selection import SelectKBest, chi2, f_regression

    if method == FeatureSelectionMethod.CHI2:
        selector = SelectKBest(chi2, k=k)
    elif method == FeatureSelectionMethod.F_REGRESSION:
        selector = SelectKBest(f_regression, k=k)
    else:
        raise ValueError("Invalid feature selection method.")

    selector.fit(x, y)
    return selector, selector.get_support(indices=True), selector.scores_


def univariate_feature_selection(df, target_col: str, k: int, selection_method: FeatureSelectionMethod):
    """
    Selects the top K features based on univariate statistical tests, adjusting for the target variable type.
    It then adds the original values of these features as new columns to the DataFrame,
    with column names reflecting the feature name and its score.
    :param target_col: Name of the target variable column.
    :param k: Number of top features to select.
    :param selection_method: Method of feature selection, based on target variable type.
    """
    import pandas as pd

    x = df.drop(columns=[target_col])
    y = df[target_col]

    # Ensure that X contains only numeric columns
    x_numeric = x.select_dtypes(include=['number'])
    y = y if selection_method == FeatureSelectionMethod.CHI2 else pd.to_numeric(y, errors='coerce').fillna(0)

    selector, support, scores = feature_selection_helper(x_numeric, y, k, selection_method)

    # Filter for selected features based on support
    selected_features = x_numeric.columns[support]

    # Score normalization for naming (optional)
    normalized_scores = [round(score) for score in scores[support]]

    # Add original values of selected features to the DataFrame with updated names
    for feature, score in zip(selected_features, normalized_scores):
        df[f'{feature}_Score_{score}'] = df[feature]

    return df


# ** Advanced Functionality **
univariate_feature_selection.worker_required = True
